import serial
import time
import sys
import os

# V1.0, June 2013

class statuserror(Exception):
   def __init__(self, value):
      self.value = value
   def __str__(self):
      return repr(self.value)


def waitbyte(sp):
   c = ''
   while (len(c) == 0):
      c = sp.read(1)
   return ord(c)


def expectstatus(sp, n):
   #print 'waiting for status...'
   c = waitbyte(sp)
   #print 'status:', c
   if (c != n):
      raise statuserror(c)


def sendcommand(sp, cmd):
   print 'command:', cmd
   sp.write(cmd)
   expectstatus(sp, 0)


def blocksum(block):
   n = 0
   for x in xrange(len(block)):
      n = n + ord(block[x])
   return n & 255;
   

def sendblock(sp, block):
   sp.write('\0')

   cxsum = 0
   rxsum = 1
   while (cxsum != rxsum):
      sp.write(block)

      # adjust for short blocks
      if len(block) != 128:
         dummy = "x" * 128 - len(block)
         sp.write(dummy)
         
      cxsum = blocksum(block)
      rxsum = waitbyte(sp)
      sp.write(chr(cxsum))

      if rxsum != cxsum:
         print 'rx: ' + `rxsum` + ' cx:' + `cxsum`
         print 'Retrying...'


def readblock(sp, nbytes):
   blocktype = waitbyte(sp)
   if blocktype == 3:
      # repeat block
      d = waitbyte(sp)
      return chr(d) * nbytes

   # else it's a regular block

   cxsum = 0
   rxsum = 1
   data = bytearray(nbytes)

   while cxsum != rxsum:
      cxsum = 0
      for x in xrange(nbytes):
         data[x] = waitbyte(sp)
         cxsum = cxsum + data[x]

      cxsum &= 255
      
      rxsum = waitbyte(sp)
      sp.write(chr(cxsum))

      if rxsum != cxsum:
         print 'rx: ' + `rxsum` + ' cx:' + `cxsum`
         print 'Retrying...'

   return data


def main(args):
   filename = args[[0]

   # default port number for charlie's laptop
   port = 4
   for x in xrange(len(args)):
      if args[x][:6] == '/port:':

   print 'Writing to', filename
   image = open(filename, 'wb')

   try:
      sp = serial.Serial('COM'+`port`, 9600, timeout=0, rtscts=True)
   except:
      print 'Error opening serial port COM' + `port`
      return

   # OK, GO!

   sendcommand(sp, '!S\x00')
   sp.write('!r\x00\x28\x0a\x0a')

   tracksleft = 40
   while (tracksleft > 0):
      print 'track', `40-tracksleft`,
      sp.write(0)                   # i'm ready for data, continue
      data = readblock(sp, 10)      # read sector statuses
      for x in xrange(10):
         print repr(data[x]),
      print
      for x in xrange(10):          # read sectors
         data = readblock(sp, 512)
         image.write(data)
         print '.',
      print 'OK.'

      tracksleft = tracksleft - 1

   sp.write(2)
   image.close()

if __name__ == '__main__':
   main(sys.argv[1:])
