import serial
import time
import sys
import os

# V1.0, June 2013

def printopts():
   print 'Usage:\n  etSendFile.py path_to_file (options)'
   print 'Options:\n  /port:[n]  :  specify serial port number e.g. 4 = COM4'
   return 1


class statuserror(Exception):
   def __init__(self, value):
      self.value = value
   def __str__(self):
      return repr(self.value)


def waitbyte(sp):
   c = ''
   while (len(c) == 0):
      c = sp.read(1)
   return ord(c)


def expectstatus(sp, n):
   #print 'waiting for status...'
   c = waitbyte(sp)
   #print 'status:', c
   if (c != n):
      raise statuserror(c)


def sendcommand(sp, cmd):
   print 'command:', cmd
   sp.write(cmd)
   expectstatus(sp, 0)


def blocksum(block):
   n = 0
   for x in xrange(len(block)):
      n = n + ord(block[x])
   return n & 255;
   

def sendblock(sp, block):
   sp.write('\0')

   cxsum = 0
   rxsum = 1
   while (cxsum != rxsum):
      sp.write(block)

      # adjust for short blocks
      if len(block) != 128:
         dummy = "x" * 128 - len(block)
         sp.write(dummy)
         
      cxsum = blocksum(block);
      rxsum = waitbyte(sp)
      sp.write(chr(cxsum))

      if rxsum != cxsum:
         print 'rx: ' + `rxsum` + ' cx:' + `cxsum`
         print 'Retrying...'


def main(args):
   if len(args) == 0:
      return printopts()

   # default port number for charlie's laptop
   port = 4

   for x in xrange(len(args)):
      if args[x][:6] == '/port:':
         port = int(args[x][6:])

   filename = args[0]

   if !os.path.isfile(filename):
      print 'Requested file ' + filename + ' is not a file.'
      return
      
   try:
      data = open(filename, 'rb').read()
   except IOError:
      print 'Requested file ' + filename + ' does not exist.'
      return

   fileparts = os.path.split(filename)
   if len(fileparts) == 2:
      filename = fileparts[1]

   #convert filename to eintrans version
   # name starts with 0, the drive specifier.
   # we then need at most 8 characters of name, padded with spaces if less,
   # finally we'll crown it with the extension, all uppercase
   fileparts = filename.split('.')
   root = (fileparts[0] + '        ')[:8]
   ext = (fileparts[1] + '   ')[:3]
   einyname = ('0' + root + ext).upper()

   try:
      sp = serial.Serial('COM'+`port`, 9600, timeout=0, rtscts=True)
   except:
      print 'Error opening serial port COM' + `port`
      return

   # OK, GO!
   
   sendcommand(sp, '!S\x00')
   sendcommand(sp, '!W0' + einyname)

   idx = 0
   remaining = len(data)
   while (remaining > 0):
      print 'sending block', (idx/128)+1,'/',(len(data)/128)

      bsize = 128;
      if remaining < 128:
         bsize = remaining

      block = data[idx:idx+bsize]

      sendblock(sp, block)
      expectstatus(sp,0)

      idx = idx + bsize
      remaining -= bsize

      if (remaining != 0):
         sp.write('\x00')
      else:
         sp.write('\x02')



if __name__ == '__main__':
   main(sys.argv[1:])
